﻿using System;

namespace Nyerguds.Util.UI
{
    /// <summary>
    /// To be implemented in subclasses as "class SubClass : DataSourceListItem&lt;Subclass,EncapsulatedType&gt;"
    /// so the static functions can have the implementing class as return type.
    /// </summary>
    /// <typeparam name="T"></typeparam>
    /// <typeparam name="U"></typeparam>
    public class DataSourceListItem<T,U> where T: DataSourceListItem<T,U>, new() where U : class
    {
        public U InternalObject { get; set; }
        public String StringValue { get; set; }

        public DataSourceListItem()
        {
        }

        public override String ToString()
        {
            if (this.InternalObject == null && this.StringValue == null)
            {
                return "null";
            }
            else if (this.StringValue != null)
            {
                return StringValue;
            }
            else
            {
                String customString = GetObjectString();
                if (customString != null)
                    return customString;
            }
            return this.InternalObject.ToString();
        }

        /// <summary>
        /// Extra methods to create a custom substring from the encapsulated oblect. Override in implementing subclass.
        /// </summary>
        /// <returns>The custom string representation of the encapsulating object, or null to use the standard</returns>
        protected virtual String GetObjectString()
        {
            return null;
        }

        public static T CreateObject(U internalObject)
        {
            T newobj = new T();
            newobj.InternalObject = internalObject;
            return newobj;
        }

        public static T[] GetList(U[] objects)
        {
            return GetList(objects, "null");
        }

        /// <summary>
        /// For handling objects retrieved from a data source; they can't be handled as the implementing subclass since casting them would give an exception.
        /// </summary>
        /// <param name="obj"></param>
        /// <returns></returns>
        public static T GetAsHandledType(Object obj)
        {
            if (obj is T)
                return (T)obj;
            else throw new InvalidCastException();
        }

        public static T[] GetList(U[] objects, String nullDefault)
        {
            if (objects == null)
                return new T[0];
            T[] output = new T[objects.Length];
            for (Int32 i = 0; i < objects.Length; i++)
            {
                U obj = objects[i];
                output[i] = CreateObject(obj);
                if (obj == null)
                    output[i].StringValue = nullDefault;
            }
            return output;
        }
    }

    /// <summary>
    /// Simple implementation that can be used on the fly.
    /// </summary>
    /// <typeparam name="T">Any class type</typeparam>
    public class DSListItem<T> : DataSourceListItem<DSListItem<T>, T> where T: class
    {

    }
}
